# encoding: UTF-8
=begin ########################################################################
= Copyright (c) 2020 AgiliCity d.o.o.. All Rights Reserved.
=
= By using this software, you agree to respect Terms and Conditions, End User
= License Agreement and other related documents that can be found on our
= website (https://modelur.com/legal/).
=
= Make sure this file is not encoded by Ruby Encoder:
= RubyEncoder:DO_NOT_ENCODE
=end ##########################################################################


require 'sketchup'
require 'extensions'
require "rbconfig"

module Modelur
  MODELUR_PLATFORM ||= begin
                         host_os = RbConfig::CONFIG["host_os"]

                         case host_os
                         when /mswin|msys|mingw|cygwin|bccwin|wince|emc/
                           :windows
                         when /darwin|mac os/
                           :osx
                         else
                           raise "Unsupported OS: #{host_os.inspect}"
                         end
                       end


  def self.rg_loader_paths(install_path)
    old_folders = Dir["#{install_path}/*rgloader*/"].sort + Dir["#{install_path}/rgloader/rgloader*/"].sort
    current_folder = old_folders.pop
    old_folders.reject! { |dir| current_folder.start_with?(dir) }
    [current_folder, old_folders]
  end

  private_class_method :rg_loader_paths

  def self.remove_paths(dirs_to_remove)
    return if dirs_to_remove.empty?
    require 'fileutils' unless defined? FileUtils
    dirs_to_remove.each { |dir| FileUtils.remove_dir(dir) }
  rescue StandardError => e
    print e.message
    e.backtrace.each { |err| print "\n" + err }
  end

  private_class_method :remove_paths

  def self.install_path
    plugins_path = Sketchup.find_support_file('Plugins')
    modelur_path = File.join(plugins_path, 'Modelur')
    return modelur_path if File.exist? modelur_path
    file = __FILE__.dup
    file.force_encoding('UTF-8') if file.respond_to?(:force_encoding)
    modelur_path = File.join(File.dirname(file), 'Modelur')
    modelur_path = File.realpath(modelur_path) if File.respond_to? :realpath
    modelur_path
  end

  private_class_method :install_path

  MODELUR_INSTALL_PATH = install_path unless defined? MODELUR_INSTALL_PATH
  MODELUR_USER_DEFAULTS_PATH = File.join(MODELUR_INSTALL_PATH, 'user').freeze

  loader_dir, old_loader_dirs = rg_loader_paths(MODELUR_INSTALL_PATH)
  remove_paths(old_loader_dirs)

  MODELUR_RUBY_VERSION = RUBY_VERSION.to_i unless defined? MODELUR_RUBY_VERSION
  SKETCHUP_VERSION = Sketchup.version.to_i unless defined? SKETCHUP_VERSION
  SKETCHUP_VERSION_MIN = 17 unless defined? SKETCHUP_VERSION_MIN
  SKETCHUP_VERSION_MAX = 23 unless defined? SKETCHUP_VERSION_MAX
  if MODELUR_PLATFORM == :osx
    TMP_DIR = ENV['TMPDIR'] unless defined? TMP_DIR
  else
    TMP_DIR = ENV['TMP'] unless defined? TMP_DIR
  end
  MODELUR_TIMEOUT = 3
  MODELUR_VERSION_EXTERNAL = '2022.12.03'

  loader_file = File.join(loader_dir, 'loader.rb')
  loader_dir.force_encoding('UTF-8') if loader_dir.respond_to?(:force_encoding)
  loader_file.force_encoding('UTF-8') if loader_file.respond_to?(:force_encoding)
  raise "\n\n####################################\nModelur could not be loaded on this computer.\nPlease contact Modelur team at support@modelur.com.\n####################################\n\n" unless File::exist?(loader_file)
  RG_LOADER_DIR = loader_dir unless defined? RG_LOADER_DIR
  RG_LOADER_FILE = loader_file unless defined? RG_LOADER_FILE
  require RG_LOADER_FILE unless SKETCHUP_VERSION > SKETCHUP_VERSION_MAX


  if SKETCHUP_VERSION < SKETCHUP_VERSION_MIN
    UI.messagebox("Modelur can not run on SketchUp versions below 20#{SKETCHUP_VERSION_MIN}.\n\nPlease contact us at support@modelur.com if you need a version that works on older versions of SketchUp.", MB_OK)
  elsif SKETCHUP_VERSION > SKETCHUP_VERSION_MAX
    UI.messagebox("Modelur #{MODELUR_VERSION_EXTERNAL} can not run on SketchUp versions above 20#{SKETCHUP_VERSION_MAX}.", MB_OK)
  else
    @modelurExtension = SketchupExtension.new "Modelur", File.join("Modelur", "bootstrap")
    @modelurExtension.creator = "AgiliCity"
    @modelurExtension.copyright = "©2023 AgiliCity"
    @modelurExtension.version = "#{MODELUR_VERSION_EXTERNAL}, valid until Dec 31st 2024."
    modelurDescription = "Modelur is SketchUp Extension for parametric urban design. For more information, please visit https://modelur.com."
    @modelurExtension.description = modelurDescription

    Sketchup.register_extension @modelurExtension, true
  end
end
